<?php
/*
    Plugin Name: Пример плагина оплаты
    Description: Пример описания плагина оплаты
    Author: Best Developer
    Version: 0.0.0
    Edition: PAYMENT-EXAMPLE
*/

new PaymentExmple;

class PaymentExmple {
    public static $pluginName = 'payment-example'; // Одновременно название папки плагина, code оплаты и плагин оплаты

    public function __construct() {
        // Действие при активации плагина
        mgActivateThisPlugin(__FILE__, [__CLASS__, 'activate']);

        // Отрисовка формы оплаты (после оформления заказа)
        mgAddAction(self::$pluginName, [__CLASS__, 'getOrderForm'], 1);

        // Обработка webhooks (запросы, которые приходят с банков при совершении какого-то события)
        mgAddAction('Models_Payment_handleRequest', [__CLASS__, 'webhookIntercept'], 1);
    }

    // Этот метод срабатывает при активации плагина (При включении или активации оплаты)
    public function activate() {
        // Проверяем, существует ли оплата этого плагина
        $currentPayment = Models_Payment::getPaymentByPlugin(self::$pluginName);
        // Если нет, то созадём
        if (!$currentPayment) {
            // Имя / Публичное имя оплаты
            $name = 'Пример плагина оплаты';

            // Сразу добавляем иконку из плагина
            $icon = SITE.'/mg-plugins/'.self::$pluginName.'/src/icon.png';

            // Параметры оплаты
            $defaultParams = [
                [
                    // Имя опции (для доступа в коде)
                    'name' => 'example',
                    // Название опции (отображается в редактировании оплаты)
                    'title' => 'Тестовая настройка',
                    // Тип настройки
                    //
                    // text     - обычный текстовый input
                    // crypt    - input; типа password, в базе значение этой настройки хранится в зашифрованном виде
                    // checkbox - чекбокс; настройка может хранить булевые значения true/false
                    // select - select; обязательно принимает одно из значений прописанных в options
                    // multiselect - select с атрибутом multi; может принимать несколько или ни одного значения из массива options
                    'type' => 'multiselect',
                    // значение настройки (кроме multiselect)
                    // 'value' => 'some value',
                    // массив выбранных значений, указывается в настройке типа multiselect
                    'values' => ['val1, val4'],
                    // возможные значения, указываются в настройках типа select или multiselect
                    'options' => [
                        [
                            // Выводимое название опции
                            'title' => 'Значение 1',
                            // Значение опции
                            'value' => 'val1',
                        ],
                        [
                            'title' => 'Значение 2',
                            'value' => 'val2',
                        ],
                        [
                            'title' => 'Значение 3',
                            'value' => 'val3',
                        ],
                        [
                            'title' => 'Значение 4',
                            'value' => 'val4',
                        ],
                    ],
                ]
            ];

            // Список ссылок, который выводится в окне редактирования оплаты
            $urls = [
                [
                    // Тип ссылки. В зависимости от него ссылка будет обрамлена в соответствующий цвет
                    // info - голубая, success - зелёная, fail - красная, warning - желтая
                    'type' => 'info',
                    // Название ссылки
                    'title' => 'Webhook URL',
                    // Ссылка
                    'link' => SITE.'/payment?payment='.self::$pluginName,
                ],
            ];

            Models_Payment::addPayment(
                // Код оплаты - уникальный для каждой оплаты строковый идентификатор
                // Для удобства рекомендуем использовать название папки плагина
                self::$pluginName,
                // Название оплаты для менеджера (Отображается в административной панели)
                $name,
                // Название оплаты для покупателя (Отображается в публичной части сайта)
                $name,
                // Привязанный плагин (Указывается название папки с плагином)
                self::$pluginName,
                // Список параметров оплат (доступны в редактировании оплаты)
                $defaultParams,
                // Иконка оплаты (ссылка)
                $icon,
                // Поддержка логирования. 0 - не поддерживается, 1 - поддерживается
                // Имеется ввиду логирование различных событий в плагине оплаты
                // с использование метода Models_Payment::loger
                1,
                // Массив ссылок для оплаты. Это могут быть ссылки возврата по завершению оплаты
                // или ссылки для webhooks (нотификации). Выводятся в редактировании оплаты.
                $urls,
                // Скидка/Наценка на способ оплаты
                0,
                // Кому доступна оплата all - всем, fiz - физическим лицам, yur - юридическим лицам,
                'all',
                // Сделать оплату неактивной сразу после подключения
                // Это необходимо чтобы в публичной части не отображалась оплата до её настройки
                true
            );
        }
    }

    // Этот метод срабатывает при отрисовки формы оплаты по завершению оформления заказа
    public static function getOrderForm($args) {
        $result = $args['result'];

        // Получаем данные о заказе
        $orderId = $args['args'][1];
        $orderModel = new Models_Order();
        $orders = $orderModel->getOrder('`id` = '.DB::quoteInt($orderId));
        $order = $orders[$orderId];
        // При работе плагинов скидок или снижения цен,
        // стоимость товаров или стоимость заказа может иметь число с более двух цифр после запятой,
        // чтобы устранить этот эффект, содержимое заказа рекомендуется получать через метод getCorrectOrderContent,
        // а сумму заказа округлять через round($order['summ'], 2)
        // перед передачей параметров заказа в банк
        $orderContent = $orderModel->getCorrectOrderContent($order);

        // Генерируем вёрстку из views/form.php
        ob_start();
        require('views/form.php');
        $result = ob_get_contents();
        ob_end_clean();

        return $result;
    }

    // Обработка запросов на контроллер payment. (Как правило это webhooks от банков)
    public static function webhookIntercept($args) {
        $result = $args['result'];
        // Проверяем что запрос адресован этому плагину
        if ($_GET['payment'] !== 'payment-example') {
            return $result;
        }

        if (
            empty($_POST['orderId']) ||
            empty($_POST['sum'])
        ) {
            echo 'ERR';
            exit;
        }

        // По возможности логируем взаимодействие плагина с банковской системой
        Models_Payment::loger(self::$pluginName, __CLASS__.'_'.__FUNCTION__, $_POST);

        $orderId = $_POST['orderId'];
        $sum = $_POST['sum'];
        $status = $_POST['status'];

        // Достаём данные о заказе, который пришёл в webhook
        // Обязательно проверяем что совпадают сумма в webhook и сумма стоимостей товаров заказа и стоимости доставки
        $orderModel = new Models_Order();
        $orders = $orderModel->getOrder('`id` = '.DB::quoteInt($orderId).' AND ROUND(`summ` + `delivery_cost`) = '.DB::quoteFloat($sum));
        if (empty($orders)) {
            echo 'ERR';
            exit;
        }
        $order = array_shift($orders);

        // Получаем оплату плагина
        $payment = Models_Payment::getPaymentByPlugin(self::$pluginName);

        if (empty($payment)) {
            echo 'ERR';
            exit;
        }

        // Проверяем оплата заказа всё ещё та, что привязана к плагину
        if (intval($order['payment_id']) !== intval($payment['id'])) {
            echo 'ERR';
            exit;
        }

        // Вероятнее всего тут будут ещё проверки, вроде подписи запроса
        // Или дополнительного запроса на информацию о заказе в банк

        // Если вебхук сообщает что заказ успешно оплачен, то вызываем actionWhenPayment
        if ($status === 'SUCCESS') {
            Controllers_Payment::actionWhenPayment(
                [
                    'paymentOrderId' => $order['id'],
                    'paymentAmount' => $order['summ'] + $order['delivery_cost'],
                    'paymentID' => $payment['id']
                ]
            );
        }
        echo 'OK';
        exit;
    }
}